@@ -1,9 +1,12 @@
 #!/bin/sh
+# A Kino script that invokes growisofs on a generated dvdauthor xml file.
+
+DVD_DEVICE="${DVD_DEVICE:-/dev/dvd}"
 
 usage()
 {
 	# Title
-	echo "Title: Burn to /dev/dvd with growisofs"
+	echo "Title: Burn to $DVD_DEVICE with growisofs"
 
 	# Usable?
 	which dvdauthor > /dev/null && \
@@ -18,7 +21,7 @@ execute()
 	label=`basename "$2"`
 
 	dvdauthor -o "$output" -x "$xml" && \
-	growisofs -dvd-compat -Z /dev/dvd -dvd-video -V "$label" "$output"
+	growisofs -dvd-compat -Z "$DVD_DEVICE" -dvd-video -V "$label" "$output"
 }
 
 [ "$1" = "--usage" ] || [ -z "$1" ] && usage "$@" || execute "$@"
