@@ -38,23 +38,23 @@ execute()
 	
 	# Determine audio codec (MP3 if avail)
 	acodec="mp2"
-	mp3=0
-	mp3=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*EA.*mp3)" | grep mp3 | wc -l`
-	[ "$mp3" -gt 0 ] && acodec="mp3"
+	mp3test=0
+	mp3test=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*EA.*mp3)" | grep mp3 | wc -l`
+	[ "$mp3test" -gt 0 ] && acodec="$mp3"
 
 	# Run the command
 	case "$profile" in 
-		"0" ) 	ffmpeg -f dv -i pipe: $hq $interlace -g 300 -vtag DIVX -s $normalisation \
+		"0" ) 	ffmpeg -threads $threads -f dv -i pipe: $hq $interlace -g 300 -vtag DIVX -s $normalisation \
 			-aspect $aspect -qscale 2 \
 			-acodec "$acodec" -ab 192$audio_kilo -y "$file".avi ;;
-		"1" ) 	ffmpeg -f dv -i pipe: $hq $progressive -g 300 -vtag DIVX -s $full_res \
+		"1" ) 	ffmpeg -threads $threads -f dv -i pipe: $hq $progressive -g 300 -vtag DIVX -s $full_res \
 			-aspect $aspect -qscale 2 $qpel \
 			-acodec "$acodec" -ab 192$audio_kilo -y "$file".avi ;;
-		"2" ) 	ffmpeg -f dv -i pipe: $hq $progressive -g 300 -vtag DIVX -s $med_res \
+		"2" ) 	ffmpeg -threads $threads -f dv -i pipe: $hq $progressive -g 300 -vtag DIVX -s $med_res \
 			-aspect $aspect -qscale 4 -acodec "$acodec" -ar 44100 -ab 128$audio_kilo -y "$file".avi ;;
-		"3" ) 	ffmpeg -f dv -i pipe: $hq $progressive -g 120 -g 300 -vtag DIVX -s $med_res \
+		"3" ) 	ffmpeg -threads $threads -f dv -i pipe: $hq $progressive -g 120 -g 300 -vtag DIVX -s $med_res \
 			-aspect $aspect -b 500$kilo -acodec "$acodec" -ar 32000 -ab 64$audio_kilo -y "$file".avi ;;
-		"4" ) 	ffmpeg -f dv -i pipe: $hq $progressive -r 12 -g 120 -vtag DIVX -s $low_res \
+		"4" ) 	ffmpeg -threads $threads -f dv -i pipe: $hq $progressive -r 12 -g 120 -vtag DIVX -s $low_res \
 			-aspect $aspect -b 96$kilo -acodec "$acodec" -ac 1 -ar 22050 -ab 32$audio_kilo -y "$file".avi ;;
 	esac
 }
