@@ -93,7 +93,7 @@ execute()
 					mplex -v 0 -f 8 -o "$VOB" "$M2V" "$AC3" &
 
 					XML=`mktemp $file-dvdauthor.XXXXXX`
-					CHAPTERS="`$get_chapters -f $frame_rate -x $smil_file`"
+					CHAPTERS="`$get_chapters -f $frame_rate -x \"$smil_file\"`"
 					echo "<?xml version=\"1.0\"?>" > "$XML"
 					echo "<dvdauthor><vmgm><menus><video/><audio/><subpicture lang=\"en\"/></menus></vmgm><titleset><titles><pgc pause=\"0\">" >> "$XML"
 					echo "<vob file=\"$VOB\" chapters=\"$CHAPTERS\" pause=\"0\"/>" >> "$XML"
