@@ -33,7 +33,7 @@ execute()
 	audio_bitrate=224$audio_kilo
 
 	# Run the command
-	ffmpeg -f dv -i - -f vcd -deinterlace -r "$normalisation" -s "$size" -b "$video_bitrate" -acodec mp2 -ab "$audio_bitrate" -y "$file".mpeg
+	ffmpeg -threads $threads -f dv -i - -f vcd -deinterlace -r "$normalisation" -s "$size" -b "$video_bitrate" -acodec mp2 -ab "$audio_bitrate" -y "$file".mpeg
 }
 
 [ "$1" = "--usage" ] || [ -z "$1" ] && usage "$@" || execute "$@"
