@@ -86,35 +86,35 @@ execute()
 		"0" ) 	$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
 				video/x-dv,systemstream=\(boolean\)false ! $dvdec ! \
 				ffmpegcolorspace ! $deinterlace \
-				theoraenc quality=63 ! oggmux name=mux ! filesink location="$file".ogg demux. ! \
+				theoraenc quality=63 ! oggmux name=mux ! filesink location="$file".ogv demux. ! \
 				audio/x-raw-int ! queue ! audioconvert ! vorbisenc quality=1.0 ! mux. ;;
 
 		"1" ) 	$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
 				video/x-dv,systemstream=\(boolean\)false ! $dvdec ! \
 				ffmpegcolorspace ! videocrop left=8 right=8 ! $deinterlace \
 				videoscale method=1 ! video/x-raw-yuv,width=$full_res_x,height=$full_res_y,pixel-aspect-ratio=\(fraction\)1/1 ! \
-				theoraenc quality=44 ! oggmux name=mux ! filesink location="$file".ogg demux. ! \
+				theoraenc quality=44 ! oggmux name=mux ! filesink location="$file".ogv demux. ! \
 				audio/x-raw-int ! queue ! audioconvert ! vorbisenc quality=0.3 ! mux. ;;
 
 		"2" ) 	$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
 				video/x-dv,systemstream=\(boolean\)false ! $dvdec ! videorate ! \
 				video/x-raw-yuv ! ffmpegcolorspace ! videocrop left=8 right=8 ! \
 				videoscale method=0 ! video/x-raw-yuv,width=$med_res_x,height=$med_res_y,pixel-aspect-ratio=\(fraction\)1/1 ! \
-				theoraenc quality=44 ! oggmux name=mux ! filesink location="$file".ogg demux. ! \
+				theoraenc quality=44 ! oggmux name=mux ! filesink location="$file".ogv demux. ! \
 				audio/x-raw-int ! queue ! audioconvert ! vorbisenc quality=0.3 ! mux. ;;
 
 		"3" ) 	$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
 				video/x-dv,systemstream=\(boolean\)false ! $dvdec ! videorate ! \
 				video/x-raw-yuv,framerate=$halffps ! ffmpegcolorspace ! videocrop left=8 right=8 ! \
 				videoscale method=0 ! video/x-raw-yuv,width=$med_res_x,height=$med_res_y,pixel-aspect-ratio=\(fraction\)1/1 ! \
-				theoraenc bitrate=500 keyframe-freq=200 ! oggmux name=mux ! filesink location="$file".ogg demux. ! \
+				theoraenc bitrate=500 keyframe-freq=200 ! oggmux name=mux ! filesink location="$file".ogv demux. ! \
 				audio/x-raw-int ! queue ! audioresample ! audio/x-raw-int,rate=32000 ! audioconvert ! vorbisenc max-bitrate=64000 ! mux. ;;
 
 		"4" ) 	$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
 				video/x-dv,systemstream=\(boolean\)false ! $dvdec ! videorate ! \
 				video/x-raw-yuv,framerate=$halffps ! ffmpegcolorspace ! videocrop left=8 right=8 ! \
 				videoscale method=0 ! video/x-raw-yuv,width=$low_res_x,height=$low_res_y,pixel-aspect-ratio=\(fraction\)1/1 ! \
-				theoraenc bitrate=84 keyframe-freq=150 ! oggmux name=mux ! filesink location="$file".ogg demux. ! \
+				theoraenc bitrate=84 keyframe-freq=150 ! oggmux name=mux ! filesink location="$file".ogv demux. ! \
 				audio/x-raw-int ! queue ! audioresample ! audio/x-raw-int,rate=22050 ! audioconvert ! vorbisenc max-bitrate=44000 ! mux. ;;
 	esac
 }
