@@ -7,7 +7,7 @@ usage()
 	echo "Title: rawplay playback"
 
 	# Usable?
-	which rawplay > /dev/null
+	which rawplay > /dev/null 2>&1
 	[ $? -eq 0 ] && echo Status: Active || echo Status: Inactive
 
 	# Type
