@@ -1,5 +1,6 @@
 /*
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -344,7 +345,6 @@ on_main_window_visibility_notify_event ( GtkWidget *widget,
         GdkEvent *event,
         gpointer user_data )
 {
-	printf( ">> on_main_window_visibility_notify_event\n" );
 	return FALSE;
 }
 
@@ -354,7 +354,6 @@ on_main_window_event ( GtkWidget *widget,
                        GdkEvent *event,
                        gpointer user_data )
 {
-	printf( ">> Event = %d\n", event->type );
 	//visibilityChanged();
 	return FALSE;
 }
@@ -365,8 +364,6 @@ on_main_window_map_event ( GtkWidget *widget,
                            GdkEvent *event,
                            gpointer user_data )
 {
-
-	printf( ">> on_main_window_map_event\n" );
 	visibilityChanged( TRUE );
 	return FALSE;
 }
@@ -377,7 +374,6 @@ on_main_window_unmap_event ( GtkWidget *widget,
                              GdkEvent *event,
                              gpointer user_data )
 {
-	printf( ">> on_main_window_unmap_event\n" );
 	visibilityChanged( FALSE );
 	return FALSE;
 }
@@ -429,7 +425,6 @@ on_hscale_shuttle_button_release_event ( GtkWidget *widget,
         GdkEventButton *event,
         gpointer user_data )
 {
-	printf("button-release\n"), fflush(stdout);
 	doShuttle = FALSE;
 	return FALSE;
 }
@@ -715,7 +710,7 @@ on_preferences_dialog_help_button_clicked
 	int page = gtk_notebook_get_current_page( GTK_NOTEBOOK( lookup_widget( GTK_WIDGET( button ), "notebook" ) ) );
 	char s[ 14 ];
 	s[ 13 ] = '\0';
-	snprintf( s, 13, "prefs.html#%d", page );
+	snprintf( s, 13, "prefs%d", page );
 	showHelp( s );
 }
 
