@@ -1,3 +1,22 @@
+/*
+* Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or
+* (at your option) any later version.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* GNU General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software Foundation,
+* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+*/
+
 #ifndef _CALLBACKS_H
 #define _CALLBACKS_H
 
@@ -900,6 +919,68 @@ void
 on_expander_properties_activate        (GtkExpander     *expander,
                                         gpointer         user_data);
 
+#include <gtk/gtk.h>
+
+void
+on_togglebutton_magick_start_toggled ( GtkToggleButton *togglebutton,
+                                       gpointer user_data );
+
+
+gboolean
+on_hscale_transition_start_button_press_event
+( GtkWidget *widget,
+  GdkEventButton *event,
+  gpointer user_data );
+
+gboolean
+on_hscale_transition_start_button_release_event
+( GtkWidget *widget,
+  GdkEventButton *event,
+  gpointer user_data );
+
+gboolean
+on_hscale_transition_end_button_press_event
+( GtkWidget *widget,
+  GdkEventButton *event,
+  gpointer user_data );
+
+gboolean
+on_hscale_transition_end_button_release_event
+( GtkWidget *widget,
+  GdkEventButton *event,
+  gpointer user_data );
+
+void
+on_button_magick_file_clicked          (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_spinbutton_magick_start_value_changed
+                                        (GtkSpinButton   *spinbutton,
+                                        gpointer         user_data);
+
+void
+on_spinbutton_magick_end_value_changed (GtkSpinButton   *spinbutton,
+                                        gpointer         user_data);
+
+gboolean
+on_entry_magick_start_focus_out_event  (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data);
+
+void
+on_entry_magick_start_activate         (GtkEntry        *entry,
+                                        gpointer         user_data);
+
+gboolean
+on_entry_magick_end_focus_out_event    (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data);
+
+void
+on_entry_magick_end_activate           (GtkEntry        *entry,
+                                        gpointer         user_data);
+
 #ifdef __cplusplus
 }
 #endif
