@@ -46,6 +46,7 @@ extern "C"
 #include <sys/time.h>
 #include <gtk/gtk.h>
 #include <math.h>
+#include <ctype.h>
 
 #include "callbacks.h"
 #include "support.h"
@@ -71,12 +72,13 @@ static void getDroppedFiles( GtkWidget * w, GdkDragContext * context,
 		/* For each URL we are passed */
 		while ( tmp )
 		{
-			if ( strcmp( *tmp, "" ) == 0 )
+			if ( *tmp == 0 || strcmp( *tmp, "" ) == 0 )
 				break;
 			if ( !strncmp( *tmp, "file:", 5 ) )
 			{
 				argv[ 1 ] = *tmp + 5;
-				argv[ 1 ][ strlen( argv[ 1 ] ) - 1 ] = '\0';
+				if ( isspace( argv[ 1 ][ strlen( argv[ 1 ] ) - 1 ] ) )
+					argv[ 1 ][ strlen( argv[ 1 ] ) - 1 ] = '\0';
 				common->bulkLoad( 2, argv );
 			}
 			tmp++;
@@ -165,7 +167,7 @@ void bulkLoad( int argc, char * argv[] )
 		common->g_currentFrame = 0;
 		common->hasListChanged = TRUE;
 		common->setWindowTitle();
-		modal_message( _( "Kino has recovered from a crash and restored your undo history!" ) );
+		windowMoved();
 	}
 	else
 	{
