@@ -49,7 +49,7 @@ extern "C"
 	void insertFile( void );
 	void appendFile( void );
 	gboolean processKeyboard( GdkEventKey * event );
-	void processCommand( char * command );
+	void processCommand( const char * command );
 	void pageStart( int );
 	int moveToFrame( int );
 	int moveByFrames( int );
