@@ -1,5 +1,6 @@
 /*
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,7 +22,6 @@
 #endif
 
 #include <iostream>
-using std::cout;
 using std::cerr;
 using std::endl;
 
@@ -375,7 +375,7 @@ void Displayer::reformat( DisplayerInput inFormat, int outFormat, void *image,
 
 	//gettimeofday(&tv, NULL);
 	//long long endtime = tv.tv_sec * 1000000 + tv.tv_usec;
-	//cout << "Reformatting took " << endtime - starttime << endl;
+	//cerr << "Reformatting took " << endtime - starttime << endl;
 }
 
 /** X Displayer Constructor (retained for historical reasons - not used).
@@ -386,7 +386,7 @@ XDisplayer::XDisplayer( GtkWidget *drawingarea ) : xImage( NULL )
 	this->drawingarea = drawingarea;
 	this->canuse = true;
 
-	cout << ">> Trying X Displayer" << endl;
+	cerr << ">> Trying X Displayer" << endl;
 #if 0
 
 	memset( &shmInfo, 0, sizeof( XShmSegmentInfo ) );
@@ -462,7 +462,7 @@ XvDisplayer::XvDisplayer( GtkWidget *drawingarea, int width, int height, bool is
 		xvImage( NULL )
 {
 
-	cout << ">> Trying XVideo at " << width << "x" << height << endl;
+	cerr << ">> Trying XVideo at " << width << "x" << height << endl;
 
 	this->drawingarea = drawingarea;
 	img_width = width;
@@ -482,11 +482,11 @@ XvDisplayer::XvDisplayer( GtkWidget *drawingarea, int width, int height, bool is
 	if ( XvQueryAdaptors( display, window, &count, &adaptorInfo ) == Success )
 	{
 
-		cout << ">>> XvQueryAdaptors count: " << count << endl;
+		cerr << ">>> XvQueryAdaptors count: " << count << endl;
 		for ( unsigned int n = 0; gotPort == false && n < count; ++n )
 		{
 			// Diagnostics
-			cout << ">>> Xv: " << adaptorInfo[ n ].name
+			cerr << ">>> Xv: " << adaptorInfo[ n ].name
 			<< ": ports " << adaptorInfo[ n ].base_id
 			<< " - " << adaptorInfo[ n ].base_id +
 			adaptorInfo[ n ].num_ports - 1
@@ -504,7 +504,7 @@ XvDisplayer::XvDisplayer( GtkWidget *drawingarea, int width, int height, bool is
 
 					list = XvListImageFormats( display, port, &formats );
 
-					cout << ">>> formats supported: " << formats << endl;
+					cerr << ">>> formats supported: " << formats << endl;
 
 					for ( int i = 0; i < formats; i ++ )
 					{
@@ -574,20 +574,23 @@ XvDisplayer::XvDisplayer( GtkWidget *drawingarea, int width, int height, bool is
 			xvImage = ( XvImage * ) XvShmCreateImage( display, port, 0x32595559, 0, width, height, &shmInfo );
 
 			shmInfo.shmid = shmget( IPC_PRIVATE, xvImage->data_size, IPC_CREAT | 0777 );
-			shmInfo.shmaddr = ( char * ) shmat( shmInfo.shmid, 0, 0 );
-			xvImage->data = shmInfo.shmaddr;
-			shmInfo.readOnly = 0;
-			if ( !XShmAttach( gdk_display, &shmInfo ) )
-			{
+			if (shmInfo.shmid < 0) {
+				perror("shmget");
 				gotPort = false;
-			}
-			XSync( display, false );
-			shmctl( shmInfo.shmid, IPC_RMID, 0 );
+			} else {
+				shmInfo.shmaddr = ( char * ) shmat( shmInfo.shmid, 0, 0 );
+				xvImage->data = shmInfo.shmaddr;
+				shmInfo.readOnly = 0;
+				if ( !XShmAttach( gdk_display, &shmInfo ) )
+				{
+					gotPort = false;
+				}
+				XSync( display, false );
+				shmctl( shmInfo.shmid, IPC_RMID, 0 );
 #if 0
-
-			xvImage = ( XvImage * ) XvCreateImage( display, port, 0x32595559, pix, width , height );
+				xvImage = ( XvImage * ) XvCreateImage( display, port, 0x32595559, pix, width , height );
 #endif
-
+			}
 		}
 	}
 	else
@@ -598,7 +601,7 @@ XvDisplayer::XvDisplayer( GtkWidget *drawingarea, int width, int height, bool is
 
 XvDisplayer::~XvDisplayer()
 {
-	cout << ">> Destroying XV Displayer" << endl;
+	cerr << ">> Destroying XV Displayer" << endl;
 
 	if ( gotPort )
 	{
@@ -676,7 +679,7 @@ void GdkDisplayer::put( void *data )
 	AspectRatioCalculator calc( ( ( GtkWidget * ) drawingarea ) ->allocation.width,
 	                            ( ( GtkWidget * ) drawingarea ) ->allocation.height,
 	                            this->preferredWidth(), this->preferredHeight(),
-	                            img_height == 576, false );
+	                            img_height == 576, m_is_wide );
 
 	GdkGC *gc = gdk_gc_new( drawingarea->window );
 	GdkPixbuf *pix = gdk_pixbuf_new_from_data( image, GDK_COLORSPACE_RGB, FALSE, 8, preferredWidth(), preferredHeight(), preferredWidth() * 3, NULL, NULL );
