@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-20054 Dan Dennedy <dan@dennedy.org>
+ * Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,9 @@ protected:
 	static char file[ PATH_MAX + NAME_MAX ];
 	GtkWidget *file_entry;
 	int count;
+	double zoom;
+	bool hasFilenameChanged;
+	int count_offset;
 };
 
 #endif
