@@ -35,7 +35,7 @@ public:
 	TextBlock();
 	~TextBlock();
 
-	GdkPixbuf *getPixbuf( DVColor fg, DVColor bg, int outline, DVColor outline, bool isMarkup );
+	GdkPixbuf *getPixbuf( DVColor fg, DVColor bg, int outline, DVColor outlinecolor, bool isMarkup );
 
 	void setSpace( int p )
 	{
@@ -85,7 +85,7 @@ public:
 
 private:
 	void fillRectangle( GdkPixbuf *pixbuf, DVColor bg );
-	void drawPixbuf( GdkPixbuf *pixbuf, DVColor fg, DVColor bg, int outline, DVColor outline );
+	void drawPixbuf( GdkPixbuf *pixbuf, DVColor fg, DVColor bg, int outline, DVColor outlinecolor );
 
 	int align;
 	int padw, padh;
