@@ -1,6 +1,7 @@
 /*
 * export.cc Export Mode Base Object
 * Copyright (C) 2002 Mads Bondo Dydensborg <madsdyd@challenge.dk>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -90,13 +91,6 @@ void Export::startExport( bool preview )
 	bool supports_export = actions & EXPORT_EXPORT;
 	bool supports_pause = actions & EXPORT_PAUSE;
 
-	/* Sanity check */
-	if ( !supports_export )
-	{
-		cerr << ">>> Export::startExport: Internal inconsistency. "
-		<< "supports_export == FALSE, but exporting..." << endl;
-	}
-
 	/* Make sure the user can only click relevant buttons
 	 exportPage->exportMutex is a protection against the buttons going
 	 into a wild events generating loop. */
@@ -585,8 +579,8 @@ char * Export::formatSecs( char * buf, int size, double seconds )
 }
 
 /** innerLoopUpdate - see header file. */
-void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every,
-		const char* activity )
+void Export::innerLoopUpdate( int progress, int begin, int end, int every,
+		int currentFrame, const char* activity )
 {
 	char buf[ 512 ];
 	struct timeval tv;
@@ -595,7 +589,9 @@ void Export::innerLoopUpdate( int currentFrame, int begin, int end, int every,
 		cerr << ">>> Export::innerLoopUpdate: Error calling gettimeofday?" << endl;
 	}
 	double now = tv.tv_sec + tv.tv_usec / 1000000.0;
-	gfloat com_ratio = ( gfloat ) ( currentFrame - begin ) / ( gfloat ) ( end + 1 - begin ) ;
+	gfloat com_ratio = ( gfloat ) ( progress - begin ) / ( gfloat ) ( end + 1 - begin ) ;
+	if ( currentFrame == -1 )
+		currentFrame = progress;
 	/* Figure out how much time we have spend */
 	if ( 0 == startTime )
 	{
