@@ -40,6 +40,7 @@ using std::setfill;
 #include <assert.h>
 #include <time.h>
 #include <sys/time.h>
+#include <string.h>
 
 #include "filehandler.h"
 #include "error.h"
@@ -47,6 +48,8 @@ using std::setfill;
 #include "avi.h"
 #include "frame.h"
 
+#define TIMECODE_TO_SEC( tc ) (((((tc).hour * 60) + (tc).min) * 60) + (tc).sec)
+
 FileTracker *FileTracker::instance = NULL;
 
 FileTracker::FileTracker( ) : mode( CAPTURE_MOVIE_APPEND )
@@ -214,25 +217,26 @@ bool FileHandler::Done()
 bool FileHandler::WriteFrame( const Frame& frame )
 {
 	static TimeCode prevTimeCode = { 0, 0, -1, 0 };
-	TimeCode timeCode;
+	TimeCode timeCode = { 0, 0, 0, 0 };
 
 	/* If the user wants autosplit, start a new file if a
 	   new recording is detected. */
-	frame.GetTimeCode( timeCode );
-	int time_diff = timeCode.sec - prevTimeCode.sec;
-	bool discontinuity = prevTimeCode.sec != -1 && ( time_diff > 1 || ( time_diff < 0 && time_diff > -59 ) );
-	if ( FileIsOpen() && GetAutoSplit() == true && ( frame.IsNewRecording() || discontinuity ) )
+	int time_diff = 0;
+	if ( frame.GetTimeCode( timeCode ) )
+		time_diff = TIMECODE_TO_SEC(timeCode) - TIMECODE_TO_SEC(prevTimeCode);
+	bool discontinuity = ( prevTimeCode.sec != -1 ) && ( time_diff > 1 || time_diff < 0 );
+	if ( FileIsOpen() && GetAutoSplit() && ( frame.IsNewRecording() || discontinuity ) )
 	{
 		Close();
 	}
 
-	if ( FileIsOpen() == false )
+	if ( ! FileIsOpen() )
 	{
 
 		string filename;
 		static int counter = 0;
 
-		if ( GetTimeStamp() == true )
+		if ( GetTimeStamp() )
 		{
 			ostringstream sb, sb2;
 			struct tm	date;
@@ -321,7 +325,10 @@ bool FileHandler::WriteFrame( const Frame& frame )
 			done = !GetAutoSplit();
 		}
 	}
-    prevTimeCode.sec = timeCode.sec;
+
+	if ( !frame.GetTimeCode( prevTimeCode ) )
+		prevTimeCode.sec = -1;
+
 	return true;
 }
 
