@@ -1,6 +1,7 @@
 /*
 * filehandler.h
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -151,6 +152,7 @@ public:
 	int GetTotalFrames();
 	bool Open( const char *s );
 	int GetFrame( Frame &frame, int frameNum );
+	int16_t* SeekAudioFrame( Frame& frame, int frameNum, int& availableSize );
 	bool GetOpenDML() const;
 	void SetOpenDML( bool );
 	int GetFormat() const
@@ -170,6 +172,9 @@ protected:
 	bool isFullyInitialized;
 	int16_t *audioBuffer;
 	int16_t *audioChannels[ 4 ];
+	bool isInterleave1to1;
+	off_t audioOffset;
+	int audioBufferSize;
 };
 
 
