@@ -56,6 +56,11 @@ using std::endl;
 #include "frame.h"
 #include "preferences.h"
 
+#if LIBAVUTIL_VERSION_INT >= (50<<16)
+#define PIX_FMT_YUV422 PIX_FMT_YUYV422
+#endif
+
+
 VideoInfo::VideoInfo() : width( 0 ), height( 0 ), isPAL( false )
 {}
 
