@@ -33,6 +33,7 @@ using std::endl;
 #include <string>
 #include <stdio.h>
 #include <samplerate.h>
+#include <string.h>
 
 #include "endian_types.h"
 #include "playlist.h"
@@ -47,10 +48,10 @@ using std::endl;
 #if defined(HAVE_LIBAVCODEC)
 extern "C"
 {
-#   include <avcodec.h>
-#   include <avformat.h>
+#   include <libavcodec/avcodec.h>
+#   include <libavformat/avformat.h>
 #ifdef HAVE_SWSCALE
-#   include <swscale.h>
+#   include <libswscale/swscale.h>
 #endif
 }
 #endif
