@@ -254,7 +254,7 @@ static void gtk_enhanced_scale_realize ( GtkWidget *widget )
 
 	enhanced_scale->trough = gdk_window_new ( widget->window, &attributes, attributes_mask );
 
-	enhanced_scale->slider = malloc( enhanced_scale->num_adjustments * sizeof(*enhanced_scale->handler_id) );
+	enhanced_scale->slider = malloc( enhanced_scale->num_adjustments * sizeof(enhanced_scale->slider[0]) );
 	attributes.width = ENHANCED_SCALE_CLASS ( enhanced_scale ) ->arrow_width;
 	attributes.height = ENHANCED_SCALE_CLASS ( enhanced_scale ) ->arrow_height;
 	attributes.event_mask |= ( GDK_BUTTON_MOTION_MASK |
