@@ -1229,8 +1229,6 @@ IEEE1394Writer::~IEEE1394Writer()
 
 void IEEE1394Writer::StartThread()
 {
-	if ( m_isRunning )
-		return;
 	pthread_create( &m_thread, NULL, ThreadProxy, this );
 }
 
@@ -1422,6 +1420,11 @@ bool iec61883Writer::Open( bool isPAL )
 		m_iec61883dv = iec61883_dv_xmit_init( m_handle, isPAL, HandlerProxy, this );
 		m_isInitialised = ( m_iec61883dv != NULL );
 		//iec61883_dv_set_synch( m_iec61883dv, 1 ); this is causing hang on close
+		if ( !m_isInitialised )
+		{
+			raw1394_destroy_handle( m_handle );
+			m_handle = NULL;
+		}
 	}
 
 	return m_isInitialised;
@@ -1429,7 +1432,7 @@ bool iec61883Writer::Open( bool isPAL )
 
 bool iec61883Writer::SendFrame( Frame &frame, bool isBlocking )
 {
-	bool result = false;
+	bool result = true;
 
 	if ( m_isReset )
 	{
@@ -1458,10 +1461,22 @@ bool iec61883Writer::SendFrame( Frame &frame, bool isBlocking )
 	
 		if ( !m_isInitialised && Open( frame.IsPAL() ) )
 		{
-			m_isInitialised = ( iec61883_dv_xmit_start( m_iec61883dv, m_channel ) == 0 );
-			StartThread();
+			m_isRunning = true;
+			result = ( iec61883_dv_xmit_start( m_iec61883dv, m_channel ) == 0 );
+			if ( result )
+			{
+				StartThread();
+			}
+			else
+			{
+				m_isRunning = false;
+				m_isInitialised = false;
+				iec61883_dv_close( m_iec61883dv );
+				m_iec61883dv = NULL;
+				raw1394_destroy_handle( m_handle );
+				m_handle = NULL;
+			}
 		}
-		result = true;
 	}
 	return result;
 }
