@@ -1,6 +1,7 @@
 /*
 * ieee1394io.cc -- asynchronously grabbing DV data
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -242,10 +243,10 @@ public:
 
 	void StartThread();
 	void StopThread();
-	unsigned int WaitForAction( int seconds = 1 );
+	int WaitForAction( bool isBlocking, int seconds = 1 );
 	void TriggerAction();
 
-	virtual void SendFrame( Frame &frame ) = 0;
+	virtual bool SendFrame( Frame &frame, bool isBlocking = true ) = 0;
 	virtual bool isValid() = 0;
 	virtual void* Thread() = 0;
 
@@ -270,7 +271,7 @@ public:
 
 	iec61883Writer( int port, unsigned int channel, unsigned int nBuffers );
 	~iec61883Writer();
-	void SendFrame( Frame &frame );
+	bool SendFrame( Frame &frame, bool isBlocking = true );
 	bool isValid( void )
 	{
 		return ( !m_isInitialised || ( m_isInitialised && m_iec61883dv != NULL ) );
@@ -304,7 +305,7 @@ public:
 	dv1394Writer( string device, unsigned int channel, unsigned int nBuffers,
 	              unsigned int cip_n, unsigned int cip_d, unsigned int syt_offset );
 	~dv1394Writer();
-	void SendFrame( Frame &frame );
+	bool SendFrame( Frame &frame, bool isBlocking = true );
 	bool isValid( void )
 	{
 		return ( m_fd >= 0 );
