@@ -100,7 +100,7 @@ public:
 		GtkWidget * label = gtk_label_new( _( "Frames:" ) );
 		gtk_widget_show( label );
 
-		GtkObject *frame_counter_adj = gtk_adjustment_new( frames, 0, 999999, 1, 10, 10 );
+		GtkObject *frame_counter_adj = gtk_adjustment_new( frames, 0, 999999, 1, 10, 0 );
 		frame_counter = gtk_spin_button_new( GTK_ADJUSTMENT( frame_counter_adj ), 1, 0 );
 		gtk_spin_button_set_value( GTK_SPIN_BUTTON( frame_counter ), frames );
 		gtk_widget_show( frame_counter );
@@ -237,9 +237,9 @@ public:
 
 	void CreateFrame( uint8_t *pixels, int width, int height, double position, double frame_delta )
 	{
-		uint8_t r = start.red + guint16( ( end.red - start.red ) * position ) >> 8 ;
-		uint8_t g = start.green + guint16( ( end.green - start.green ) * position ) >> 8;
-		uint8_t b = start.blue + guint16( ( end.blue - start.blue ) * position ) >> 8;
+		uint8_t r = start.red + ( guint16( ( end.red - start.red ) * position ) >> 8 );
+		uint8_t g = start.green + ( guint16( ( end.green - start.green ) * position ) >> 8 );
+		uint8_t b = start.blue + ( guint16( ( end.blue - start.blue ) * position ) >> 8 );
 		uint8_t *p = pixels;
 		while ( p < ( pixels + width * height * 3 ) )
 		{
