@@ -1,6 +1,7 @@
 /*
 * image_transitions.cc -- RGB24 image transitions
 * Copyright (C) 2002 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2002-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,6 +22,8 @@
 #include <config.h>
 #endif
 
+#include <iostream>
+
 // Project Includes
 
 #include "image_transitions.h"
@@ -70,11 +73,6 @@ public:
 		gtk_widget_destroy( window );
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-
 	char *GetDescription( ) const
 	{
 		return _( "Switch" );
@@ -82,6 +80,10 @@ public:
 
 	void GetFrame( uint8_t *pixels, uint8_t *mesh, int width, int height, double position, double frame_delta, bool reverse )
 	{
+		GtkRange * range = GTK_RANGE( lookup_widget( window, "hscale_switch" ) );
+		GtkAdjustment *adjust = gtk_range_get_adjustment( range );
+		point = adjust->value / 100;
+		
 		if ( position >= point )
 			memcpy( pixels, mesh, width * height * 3 );
 	}
@@ -96,13 +98,6 @@ public:
 		gtk_widget_reparent( ( GTK_BIN( bin ) ) ->child, GTK_WIDGET( window ) );
 	}
 
-	void InterpretWidgets( GtkBin *bin )
-	{
-		GtkRange * range = GTK_RANGE( lookup_widget( window, "hscale_switch" ) );
-		GtkAdjustment *adjust = gtk_range_get_adjustment( range );
-		point = adjust->value / 100;
-	}
-
 	double SwitchPoint( )
 	{
 		return point;
@@ -117,7 +112,7 @@ class ImageTransitionFade : public GDKImageTransition
 public:
 	char *GetDescription( ) const
 	{
-		return _( "Fade" );
+		return _( "Dissolve" );
 	}
 
 	void GetFrame( uint8_t *io, uint8_t *mesh, int width, int height, double position, double frame_delta, bool reverse )
@@ -127,7 +122,6 @@ public:
 		uint8_t r, g, b;
 		uint8_t kr, kg, kb;
 
-		// THIS IS A HACK TO GET AROUND A SMIL HACK...
 		if ( reverse )
 			position = 1 - position;
 
@@ -171,11 +165,6 @@ public:
 		gtk_widget_destroy( window );
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-
 	char *GetDescription( ) const
 	{
 		return _( "Push Wipe" );
@@ -234,6 +223,10 @@ public:
 
 	void GetFrame( uint8_t *pixels, uint8_t *mesh, int width, int height, double position, double frame_delta, bool reverse )
 	{
+		GtkMenu * menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_wipe" ) ) ) );
+		GtkWidget *active_item = gtk_menu_get_active( menu );
+		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
+		
 		if ( type == 0 && !reverse )
 			Left( pixels, mesh, width, height, position );
 		else if ( type == 0 && reverse )
@@ -253,13 +246,6 @@ public:
 	{
 		gtk_widget_reparent( ( GTK_BIN( bin ) ) ->child, GTK_WIDGET( window ) );
 	}
-
-	void InterpretWidgets( GtkBin *bin )
-	{
-		GtkMenu * menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_wipe" ) ) ) );
-		GtkWidget *active_item = gtk_menu_get_active( menu );
-		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
-	}
 };
 
 /** 'Differences Only' transition.
@@ -292,11 +278,6 @@ public:
 		gtk_widget_destroy( window );
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-
 	char *GetDescription( ) const
 	{
 		return _( "Differences" );
@@ -304,6 +285,17 @@ public:
 
 	void GetFrame( uint8_t *io, uint8_t *mesh, int width, int height, double position, double frame_delta, bool reverse )
 	{
+		GtkMenu *menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_action" ) ) ) );
+		GtkWidget *active_item = gtk_menu_get_active( menu );
+		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
+
+		GtkColorButton *colorButton = GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker" ) );
+		gtk_color_button_get_color( colorButton, &color );
+
+		GtkRange *range = GTK_RANGE( lookup_widget( window, "hscale_sensitivity" ) );
+		GtkAdjustment *adjust = gtk_range_get_adjustment( range );
+		sensitivity = ( int ) adjust->value;
+		
 		// Differences only
 		uint8_t * k = mesh;
 		uint8_t *p = io;
@@ -347,20 +339,6 @@ public:
 	{
 		gtk_widget_reparent( ( GTK_BIN( bin ) ) ->child, GTK_WIDGET( window ) );
 	}
-
-	void InterpretWidgets( GtkBin *bin )
-	{
-		GtkMenu *menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_action" ) ) ) );
-		GtkWidget *active_item = gtk_menu_get_active( menu );
-		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
-
-		GtkColorButton *colorButton = GTK_COLOR_BUTTON( lookup_widget( window, "colorpicker" ) );
-		gtk_color_button_get_color( colorButton, &color );
-
-		GtkRange *range = GTK_RANGE( lookup_widget( window, "hscale_sensitivity" ) );
-		GtkAdjustment *adjust = gtk_range_get_adjustment( range );
-		sensitivity = ( int ) adjust->value;
-	}
 };
 
 /** Barn Door transition.
@@ -385,11 +363,6 @@ public:
 		gtk_widget_destroy( window );
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-
 	char *GetDescription( ) const
 	{
 		return _( "Barn Door Wipe" );
@@ -444,6 +417,10 @@ public:
 
 	void GetFrame( uint8_t *pixels, uint8_t *mesh, int width, int height, double position, double frame_delta, bool reverse )
 	{
+		GtkMenu * menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_door" ) ) ) );
+		GtkWidget *active_item = gtk_menu_get_active( menu );
+		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
+		
 		if ( type == 0 && !reverse )
 			VerticalOpen( pixels, mesh, width, height, position );
 		else if ( type == 0 && reverse )
@@ -463,13 +440,6 @@ public:
 	{
 		gtk_widget_reparent( ( GTK_BIN( bin ) ) ->child, GTK_WIDGET( window ) );
 	}
-
-	void InterpretWidgets( GtkBin *bin )
-	{
-		GtkMenu * menu = GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( window, "optionmenu_door" ) ) ) );
-		GtkWidget *active_item = gtk_menu_get_active( menu );
-		type = g_list_index ( GTK_MENU_SHELL ( menu ) ->children, active_item );
-	}
 };
 
 
@@ -485,22 +455,17 @@ on_optionmenu_selected ( GtkMenuItem *menu_item, gpointer user_data )
 GDKImageTransitionRepository::GDKImageTransitionRepository()
 	: selected_transition( 0 )
 {
-	printf( ">> image transition repository created\n" );
-	
 	// Register an instance of each object (adapting raw transitions to GDK transitions where necessary)
-	Register( new ImageTransitionNone() );
-	Register( new ImageTransitionSwitch() );
 	Register( new ImageTransitionFade() );
-	Register( new ImageTransitionBarWipe() );
 	Register( new ImageTransitionBarnDoorWipe() );
-
-	// Not really transitions.. hmm...
 	Register( new ImageTransitionDifferences() );
+	Register( new ImageTransitionNone() );
+	Register( new ImageTransitionBarWipe() );
+	Register( new ImageTransitionSwitch() );
 }
 
 GDKImageTransitionRepository::~GDKImageTransitionRepository()
 {
-	printf( ">> image transition repository destroyed\n" );
 	// Remove the transitions in the repository
 	for ( unsigned int index = 0; index < transitions.size(); index ++ )
 		delete transitions[ index ];
@@ -511,7 +476,7 @@ GDKImageTransitionRepository::~GDKImageTransitionRepository()
 
 void GDKImageTransitionRepository::Register( GDKImageTransition *transition )
 {
-	printf( ">>> Image Transition: %s\n", transition->GetDescription() );
+	std::cerr << ">>> Image Transition: " << transition->GetDescription() << std::endl;
 	transitions.push_back( transition );
 }
 
@@ -549,18 +514,39 @@ GDKImageTransition *GDKImageTransitionRepository::Get( ) const
 
 void GDKImageTransitionRepository::SelectionChange( )
 {
+	bool isPreviewing = false;
+	PageMagick* magick = 0;
+
+	if ( common && common->getPageMagick( ) )
+		magick = common->getPageMagick( );
+	if ( magick && magick->IsPreviewing() )
+	{
+		isPreviewing = true;
+		magick->StopPreview();
+	}
+
 	// Detach the selected Transitions widgets
 	if ( selected_transition != NULL )
 		selected_transition->DetachWidgets( container );
 
+	if ( magick )
+	{
+		magick->SetKeyFrameControllerClient(0);
+		magick->ShowCurrentStatus( magick->GetCurrentPosition(), LOCKED_KEY, false, false );
+	}
+
 	// Get the new selection
 	selected_transition = Get();
 
-	// Inform the main page of the change (should be interface driven)
-	if ( common != NULL && common->getPageMagick( ) != NULL )
-		common->getPageMagick( ) ->RefreshStatus( );
-
 	// Attach the new transition widgets
 	if ( selected_transition != NULL )
 		selected_transition->AttachWidgets( container );
+
+	if ( magick )
+	{
+		if ( isPreviewing )
+			magick->StartPreview();
+		else
+			magick->PreviewFrame();
+	}
 }
