@@ -700,14 +700,14 @@ public:
 bool DvEncoderVideoPipeYUV::OpenVideoPipe( char *pipe, int width, int height, bool interlaced )
 {
 	sigpipe_clear();
-	char * args[ 4 ];
+	const char * args[ 4 ];
 
 	args[ 0 ] = "/bin/sh";
 	args[ 1 ] = "-c";
 	args[ 2 ] = ( char * )pipe;
 	args[ 3 ] = NULL;
 
-	g_spawn_async_with_pipes( ".", args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, NULL, NULL, &m_error );
+	g_spawn_async_with_pipes( ".", (gchar**) args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, NULL, NULL, &m_error );
 
 	this->width = width;
 	this->height = height;
@@ -764,14 +764,14 @@ public:
 bool DvEncoderVideoPipePGM::OpenVideoPipe( char *pipe, int width, int height, bool interlaced )
 {
 	sigpipe_clear();
-	char * args[ 4 ];
+	const char * args[ 4 ];
 
 	args[ 0 ] = "/bin/sh";
 	args[ 1 ] = "-c";
 	args[ 2 ] = ( char * )pipe;
 	args[ 3 ] = NULL;
 
-	g_spawn_async_with_pipes( ".", args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, NULL, NULL, &m_error );
+	g_spawn_async_with_pipes( ".", (gchar**) args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, NULL, NULL, &m_error );
 	
 	this->width = width;
 	this->height = height;
