@@ -128,6 +128,7 @@ KinoCommon::KinoCommon( GtkWidget *widget ) : last_directory( "" )
 	this->hasListChanged = TRUE;
 	this->currentScene = -1;
 	this->currentPage = -1;
+	this->showMoreInfo = false;
 
 	this->component_state = VIDEO_STOP;
 	this->is_component_state_changing = false;
@@ -1375,6 +1376,12 @@ void KinoCommon::setWindowTitle( )
 		Preferences::getInstance().addRecentFile( getPlayList()->GetDocName() );
 		updateRecentFiles();
 	}
+	bool b = GetEditorBackup()->CanUndo();
+	gtk_widget_set_sensitive( lookup_widget( widget, "undo" ), b );
+	gtk_widget_set_sensitive( lookup_widget( widget, "button_undo" ), b );
+	b = GetEditorBackup()->CanRedo();
+	gtk_widget_set_sensitive( lookup_widget( widget, "redo" ), b );
+	gtk_widget_set_sensitive( lookup_widget( widget, "button_redo" ), b );
 }
 
 /** Loads the specified media file before the specified frame.
