@@ -41,7 +41,7 @@
     activated and deactivated by a notebook page.
 */
 
-typedef enum component_enum {
+enum component_enum {
     EDIT_MENU = 2,
     SCENE_LIST = 32,
     VIDEO_START_OF_MOVIE = 64,
@@ -65,7 +65,7 @@ typedef enum component_enum {
     This assigns each page an identifier that can be used as arguments
     for certain methods.
 */
-typedef enum page_enum {
+enum page_enum {
     PAGE_EDITOR,
     PAGE_CAPTURE,
     PAGE_TIMELINE,
@@ -182,7 +182,7 @@ public:
 	}
 
 	// Common actions applied to all pages
-	bool newFile( bool prompt = true );
+	bool newFile( bool prompt = true, bool isExiting = false );
 
 	// File Load and Save dialogs
 	void bulkLoad( int, char *[] );
@@ -228,7 +228,7 @@ public:
 
 	// Current page related methods
 	gboolean processKeyboard( GdkEventKey *event );
-	gboolean processCommand( char * );
+	gboolean processCommand( const char * );
 	void selectScene( int );
 	void videoStartOfMovie();
 	void videoPreviousScene();
