@@ -1,4 +1,7 @@
 /*
+* mediactrl.c -- Jog Shuttle device support
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
+*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
@@ -106,6 +109,41 @@ static struct media_ctrl_key mc_powermate_keys[] = {
 };
 
 
+/*
+	X-Keys Jog/Shuttle
+*/
+static struct  media_ctrl_key mc_x_keys[] = {
+	{ 0x102, "Button L1", MEDIA_CTRL_F1 },
+	{ 0x103, "Button L2", MEDIA_CTRL_F9 },
+	{ 0x104, "Button L3", MEDIA_CTRL_B1 },
+	{ 0x105, "Button L4", MEDIA_CTRL_B3 },
+	{ 0x106, "Button L5", MEDIA_CTRL_B5 },
+	{ 0x10a, "Button L6", MEDIA_CTRL_F2 },
+	{ 0x10b, "Button L7", MEDIA_CTRL_F10 },
+	{ 0x10c, "Button L8", MEDIA_CTRL_B2 },
+	{ 0x10d, "Button L9", MEDIA_CTRL_B4 },
+	{ 0x10e, "Button L10", MEDIA_CTRL_B6 },
+	{ 0x112, "Button C1", MEDIA_CTRL_F3 },
+	{ 0x11a, "Button C2", MEDIA_CTRL_F4 },
+	{ 0x122, "Button C3", MEDIA_CTRL_F5 },
+	{ 0x12a, "Button C4", MEDIA_CTRL_F6 },
+	{ 0x113, "Button C5", MEDIA_CTRL_F11 },
+	{ 0x11b, "Button C6", MEDIA_CTRL_F12 },
+	{ 0x123, "Button C7", MEDIA_CTRL_F13 },
+	{ 0x12b, "Button C8", MEDIA_CTRL_F14 },
+	{ 0x132, "Button R1", MEDIA_CTRL_F7 },
+	{ 0x133, "Button R2", MEDIA_CTRL_F15 },
+	{ 0x134, "Button R3", MEDIA_CTRL_B7 },
+	{ 0x135, "Button R4", MEDIA_CTRL_B9 },
+	{ 0x136, "Button R5", MEDIA_CTRL_B11 },
+	{ 0x13a, "Button R6", MEDIA_CTRL_F8 },
+	{ 0x13b, "Button R7", MEDIA_CTRL_F16 },
+	{ 0x13c, "Button R8", MEDIA_CTRL_B8 },
+	{ 0x13d, "Button R9", MEDIA_CTRL_B10 },
+	{ 0x13e, "Button R10", MEDIA_CTRL_B12 },
+	{ 0, NULL, 0 }
+};
+
 struct  media_ctrl_key *media_ctrl_get_key(struct media_ctrl *ctrl, int code, int *index)
 {
 	int i = 0;
@@ -237,6 +275,7 @@ struct media_ctrl_device supported_devices[] = {
 	{ 0x05f3, 0x0240, "Contour ShuttlePro", mc_shuttle_pro_keys, translate_contour_hid_event },
 	{ 0x0760, 0x0001, "JLCooper MCS3", mc_jlcooper_mcs3_keys, translate_compliant },
 	{ 0x077d, 0x0410, "Griffin PowerMate", mc_powermate_keys, translate_compliant },
+	{ 0x05f3, 0x0241, "X-Keys Editor", mc_x_keys, translate_contour_hid_event },
 	{ 0, 0, 0 }
 };
 
