@@ -57,7 +57,7 @@ public:
 	{
 		return FALSE;
 	}
-	virtual gboolean processCommand( char *command )
+	virtual gboolean processCommand( const char *command )
 	{
 		return FALSE;
 	}
