@@ -61,7 +61,6 @@ extern "C"
 	extern char cmd[];
 	static char lastcmd[ 256 ];
 	static gchar lastPreferenceFilename[ 512 ];
-	extern KinoCommon *common;
 	static quadlet_t avcStatus;
 	char timecode[ 256 ];
 
@@ -827,7 +826,11 @@ gboolean PageCapture::processCommand( const char *command )
 	int count = 1;
 	char real[ 256 ] = "";
 
-	strcpy( cmd, command );
+	if ( cmd != command )
+	{
+		strncpy( cmd, command, 256 );
+		cmd[ 255 ] = '\0';
+	}
 
 	switch ( sscanf( cmd, "%d%s", &count, real ) )
 	{
