@@ -76,9 +76,9 @@ public:
 	void videoPreviousScene();
 	void videoStartOfScene();
 	void videoRewind();
-	void videoBack();
+	void videoBack( int step = 1 );
 	void videoPlay();
-	void videoForward();
+	void videoForward( int step = 1 );
 	void videoFastForward();
 	void videoNextScene();
 	void videoEndOfScene();
