@@ -711,7 +711,11 @@ gboolean PageEditor::processCommand( const char *command )
 	int count = 1;
 	char real[ 256 ] = "";
 
-	strcpy( cmd, command );
+	if ( cmd != command )
+	{
+		strncpy( cmd, command, sizeof(cmd) );
+		cmd[ sizeof(cmd) - 1 ] = '\0';
+	}
 
 	switch ( sscanf( cmd, "%d%s", &count, real ) )
 	{
@@ -1201,9 +1205,10 @@ gboolean PageEditor::processCommand( const char *command )
 	else
 	{
 		// Check for invalid commands
-		if ( strlen( real ) > 5 )
+		size_t real_len = strlen(real);
+		if ( real_len > 5 )
 			cmd[ 0 ] = 0;
-		else if ( strchr( "dgy ", real[ strlen( real ) - 1 ] ) == NULL )
+		else if ( real_len >= 1 && strchr( "dgy ", real[ strlen( real ) - 1 ] ) == NULL )
 			cmd[ 0 ] = 0;
 
 		common->keyboardFeedback( cmd, "" );
