@@ -82,7 +82,7 @@ public:
 
 	// commands from the page interface
 	gboolean processKeyboard( GdkEventKey *event );
-	gboolean processCommand( char *cmd );
+	gboolean processCommand( const char *cmd );
 	void selectScene( int );
 
 	// commands defined by this interface
