@@ -243,6 +243,17 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 	return status;
 }
 
+void ExportAudio::start()
+{
+	GtkWidget* widget = lookup_widget( common->getWidget(), "menuitem_export_audio_sox" );
+	system( "which sox > /dev/null 2> /dev/null" ) ? gtk_widget_hide( widget ) : gtk_widget_show( widget );
+	widget = lookup_widget( common->getWidget(), "menuitem_export_audio_mp2enc" );
+	system( "which mp2enc > /dev/null 2> /dev/null" ) ? gtk_widget_hide( widget ) : gtk_widget_show( widget );
+	widget = lookup_widget( common->getWidget(), "menuitem_export_audio_lame" );
+	system( "which lame > /dev/null 2> /dev/null" ) ? gtk_widget_hide( widget ) : gtk_widget_show( widget );
+	widget = lookup_widget( common->getWidget(), "menuitem_export_audio_oggenc" );
+	system( "which oggenc > /dev/null 2> /dev/null" ) ? gtk_widget_hide( widget ) : gtk_widget_show( widget );
+}
 
 extern "C"
 {
