@@ -102,6 +102,9 @@ void ExportAVI::start()
 	char s[ 20 ];
 	cerr << ">> Starting ExportAVI" << endl;
 
+#ifndef HAVE_LIBQUICKTIME
+	gtk_widget_hide( GTK_WIDGET( formatQtToggle ) );
+#endif
 	switch ( Preferences::getInstance().fileFormat )
 	{
 	case AVI_DV1_FORMAT:
