@@ -385,13 +385,13 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 		deinterlaceRequest = 3;
 
 	char deinterlace[ 30 ] = "";
-	char interlace = '0';
+	char interlace[5] = "-I 0";
 	char audioOptions[ 20 ] = "";
 
 	switch ( deinterlaceRequest )
 	{
 	case 0: // None
-		interlace = '1';
+		strcpy( interlace, "-I 1" );
 		break;
 	case 1: // Internal
 		type = PIPE_VIDEO_DEINTERLACED_MJPEG;
@@ -459,7 +459,7 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 			{
 				frequency = 48000;
 				if ( !strstr( audioencode, " -ab " ) )
-					sprintf( audioOptions, "%s -ab 192", audioOptions );
+					sprintf( audioOptions, "%s -ab 192k", audioOptions );
 			}
 			else if ( strstr( audioencode, "mp2enc" ) )
 			{
@@ -476,21 +476,25 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 		if ( strstr( videopipeInput, " -b " ) || strstr( videopipeInput, "--video-bitrate" ) )
 			strcpy( bitrate, "" );
 
+		// If the interlace is specified then remove the defaults above
+		if ( strstr( videopipeInput, " -I " ) || strstr( videopipeInput, "--interlace-mode" ) )
+			strcpy( interlace, "" );
+
 		// Yuck (!) - builds the video pipe depending on data collected above...
 		if ( strcmp( scale, "" ) )
 			videopipe = g_strdup_printf(
-			         "%s yuvscaler -v 0 -O %s -n %c | %s -f %d -I %c -n %c -a %d %s -o \'%s\'.mpv",
+			         "%s yuvscaler -v 0 -O %s -n %c | %s -f %d %s -n %c -a %d %s -o \'%s\'.mpv",
 			         deinterlace,
 			         scale, isPAL ? 'p' : 'n', videopipeInput, videoformat, interlace,
 			         isPAL ? 'p' : 'n', aspectRatio ? aspectRatio : (isWide ? 3 : 2), bitrate, filename );
 		else
-			videopipe = g_strdup_printf( "%s %s -f %d -I %c -n %c -a %d %s -o \'%s\'.mpv",
+			videopipe = g_strdup_printf( "%s %s -f %d %s -n %c -a %d %s -o \'%s\'.mpv",
 			         deinterlace, videopipeInput, videoformat, interlace,
 			         isPAL ? 'p' : 'n', aspectRatio ? aspectRatio : (isWide ? 3 : 2), bitrate, filename );
 	}
 	else
 	{
-		videopipe = g_strdup_printf( "%s %s -f %d -I %c -n %c -o \'%s\'.mpv\n",
+		videopipe = g_strdup_printf( "%s %s -f %d %s -n %c -o \'%s\'.mpv\n",
 		         deinterlace, videopipeInput, videoformat, interlace,
 		         isPAL ? 'p' : 'n', filename );
 	}
