@@ -345,6 +345,9 @@ enum export_result ExportPipe::doExport( PlayList *playlist, int begin, int end,
 
 		// Get the file name
 		string filename = ( char* ) gtk_entry_get_text( GTK_ENTRY( lookup_widget( window, "entry_export_pipe_file" ) ) );
+		string baseFileName = playlist->GetBaseFileOfFrame( begin );
+		if ( filename.length() < 1 && baseFileName.length() > 0 )
+			filename = baseFileName;
 	
 		// first call to innerLoopUpdate initializes progress tracker, which
 		// we want to do because calculateAdjustedRate generates paused time.
