@@ -1,6 +1,6 @@
 /*
 * page_export_stills.cc Notebook Firewire/AVI/Still Frame Export Page Object
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -157,6 +157,8 @@ ExportStills::doExport( PlayList * playlist, int begin, int end, int every,
 			int width = frame.GetWidth();
 			if ( frame.IsWide() )
 				width = frame.IsPAL() ? 1024 : 854;
+			else
+				width = frame.IsPAL() ? 768 : 640;
 			AspectRatioCalculator calc( width, frame.GetHeight(),
 		                            frame.GetWidth(), frame.GetHeight(),
 		                            frame.IsPAL(), frame.IsWide() );
