@@ -680,6 +680,8 @@ public:
 	;
 	virtual void PreGetFrame( int keyPosition = -1 ) // -1 = automatic
 	{ }
+	virtual void PreGetFrameAudio( int keyPosition = -1 ) // -1 = automatic
+	{ }
 	;
 	virtual void GetFrame( uint8_t *pixels, int i )
 	{ }
@@ -979,6 +981,7 @@ private:
 public:
 	void Initialise( PageMagickInfo *info );
 	void PreGetFrame( int keyPosition );
+	void PreGetFrameAudio( int keyPosition );
 	void GetFrame( uint8_t *pixels, int i );
 	void Close();
 };
@@ -1048,6 +1051,16 @@ void PageMagickTransition::PreGetFrame( int keyPosition )
 	info->postFrame ++;
 }
 
+void PageMagickTransition::PreGetFrameAudio( int keyPosition )
+{
+	if ( animateKey )
+	{
+		if ( keyPosition > -1 )
+			info->postFrame = info->end + keyPosition;
+	}
+	info->postFrame ++;
+}
+
 void PageMagickTransition::GetFrame( uint8_t *pixels, int i )
 {
 	// Sanity checks ...
@@ -1085,6 +1098,10 @@ public:
 		else
 			info->postFrame ++;
 	}
+	void PreGetFrameAudio( int keyPosition )
+	{
+		PreGetFrame( keyPosition );
+	}
 };
 
 void PageMagickFilter::Initialise( PageMagickInfo *info )
@@ -1483,6 +1500,7 @@ PageMagick::PageMagick( KinoCommon *common ) :
 	last_page( 0 ),
 	rendering( false ),
 	previewing( false ),
+	repainting( false ),
 	previewPosition( 0 ),
 	keyFrameControllerClient( 0 ),
 	isGuiLocked( false ),
@@ -1966,9 +1984,6 @@ void PageMagick::AudioThread()
 				frame_number = ( int )( ( i += info->increment ) + 0.5 ) )
 			{
 				previewPosition = frame_number - info->begin;
-				gdk_threads_enter();
-				showFrameInfo( frame_number, info->end - info->begin + 1 );
-				gdk_threads_leave();
 				if ( info->reverse )
 					frame_number = info->end - ( ( int ) i - info->begin );
 
@@ -1983,7 +1998,7 @@ void PageMagick::AudioThread()
 						samples = locked_samples;
 					frames->GetFrame( NULL, 0, 0, audio_buffers, frame_number );
 					if ( gtk_toggle_button_get_active( everyButton ) )
-						image->PreGetFrame( );
+						image->PreGetFrameAudio( );
 					sound->GetFrame( audio_buffers, frame_number, samples, locked_samples );
 					PlayAudio( audio_buffers, samples, info->frequency, info->channels, info->isPAL );
 				}
@@ -1994,6 +2009,7 @@ void PageMagick::AudioThread()
 					image->GetFrame( pixels, frame_number );
 					gdk_threads_enter();
 					ShowImage( area, pixels, info->width, info->height, info->isWide );
+					showFrameInfo( ( i += info->increment ) + 0.5, info->end - info->begin + 1 );
 					gdk_flush();
 					gdk_threads_leave();
 				}
@@ -2076,10 +2092,11 @@ void PageMagick::VideoThread()
 				{
 					// Render the effect
 					frames->GetFrame( pixels, info->width, info->height, NULL, frame_number );
-					image->PreGetFrame( );
+					image->PreGetFrame( previewPosition );
 					image->GetFrame( pixels, frame_number );
 					gdk_threads_enter();
 					ShowImage( area, pixels, info->width, info->height, info->isWide );
+					showFrameInfo( frame_number, info->end - info->begin + 1 );
 					gdk_flush();
 					gdk_threads_leave();
 				}
@@ -2090,6 +2107,7 @@ void PageMagick::VideoThread()
 					infoFrame->ExtractPreviewRGB( pixels );
 					gdk_threads_enter();
 					ShowImage( area, pixels, infoFrame->GetWidth(), infoFrame->GetHeight(), infoFrame->IsWide() );
+					showFrameInfo( frame_number, info->end - info->begin + 1 );
 					gdk_flush();
 					gdk_threads_leave();
 				}
