@@ -65,7 +65,7 @@ public:
 	void showIcons( );
 	void refresh( );
 	gboolean processKeyboard( GdkEventKey *event );
-	gboolean processCommand( char *cmd );
+	gboolean processCommand( const char *cmd );
 	void timeFormatChanged();
 	GtkListStore *getModel() const
 	{
