@@ -1480,9 +1480,9 @@ gboolean PageTrim::processKeyboard( GdkEventKey *event )
   	\param cmd		command to be processed;
 */
 
-gboolean PageTrim::processCommand( char *command )
+gboolean PageTrim::processCommand( const char *command )
 {
-	int	end;
+	int end;
 	int count = 1;
 	char real[ 256 ] = "";
 
