@@ -29,6 +29,7 @@
 #include <string>
 #include <list>
 #include <map>
+#include <algorithm>
 
 using std::cerr;
 using std::endl;
@@ -47,6 +48,8 @@ using std::map;
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <string.h>
+#include <glib/gi18n.h>
 
 // local includes
 #include "playlist.h"
@@ -54,6 +57,7 @@ using std::map;
 #include "filehandler.h"
 #include "frame.h"
 #include "stringutils.h"
+#include "message.h"
 
 const xmlChar* SMIL20_NAMESPACE_HREF = reinterpret_cast< const xmlChar* >( "http://www.w3.org/2001/SMIL20/Language" );
 const string KINO_AUTOSAVE_DIR = string( getenv("HOME") ) + string( "/.kino-history" );
@@ -2363,6 +2367,17 @@ string PlayList::GetDocTitle( ) const
 	return "";
 }
 
+string PlayList::GetBaseFileOfFrame( int frameNum ) const
+{
+	char *fname = GetFileNameOfFrame( frameNum );
+	char  bfname[strlen( fname ) + 10];
+	char *ptr = strrchr( fname, '.' );
+	if ( ptr ) *ptr = 0;
+	sprintf( bfname, "%s_%06d", fname, frameNum );
+	free( fname );
+	return bfname;
+}
+
 
 /** Editor backup - holds all the previous playlists (up to a the maxUndos preference
  	value).
@@ -2495,7 +2510,7 @@ void EditorBackup::Store( PlayList *playlist, bool isPersisted )
 void EditorBackup::Undo( PlayList *playlist )
 {
 	cerr << ">>> Received request to undo from position " << position - 1 << endl;
-	if ( position > 0 )
+	if ( CanUndo() )
 	{
 		position --;
 		playlist->Delete( 0, playlist->GetNumFrames() - 1 );
@@ -2512,7 +2527,7 @@ void EditorBackup::Undo( PlayList *playlist )
 void EditorBackup::Redo( PlayList *playlist )
 {
 	cerr << ">>> Received request to recover from position " << position + 1 << endl;
-	if ( ( position + 1 ) < ( int ) backups.size() )
+	if ( CanRedo() )
 	{
 		position ++;
 		playlist->Delete( 0, playlist->GetNumFrames() - 1 );
@@ -2581,7 +2596,9 @@ bool EditorBackup::Restore( PlayList * playlist )
 		}
 		closedir( dir );
 
-		if ( !names.empty() )
+		if ( !names.empty() &&
+			( modal_confirm( GTK_STOCK_OK, NULL, _( "Kino has detected a crash. Do you want to recover?" ) )
+				== GTK_RESPONSE_ACCEPT ) )
 		{
 			std::sort( names.begin(), names.end() );
 			Clear();
@@ -2601,13 +2618,13 @@ bool EditorBackup::Restore( PlayList * playlist )
 	return result;
 }
 
-string PlayList::GetBaseFileOfFrame( int frameNum ) const
+bool EditorBackup::CanUndo( ) const
 {
-	char *fname = GetFileNameOfFrame( frameNum );
-	char  bfname[strlen( fname ) + 10];
-	char *ptr = strrchr( fname, '.' );
-	if ( ptr ) *ptr = 0;
-	sprintf( bfname, "%s_%06d", fname, frameNum );
-	free( fname );
-	return bfname;
+	return ( position > 0 );
 }
+
+bool EditorBackup::CanRedo( ) const
+{
+	return ( ( position + 1 ) < ( int ) backups.size() );
+}
+
