@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2001-2009 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -165,7 +165,7 @@ Preferences::Preferences() : phyID( -1 )
 	get( syt_offset, "syt_offset", "0" );
 	get( preview_capture, "preview_capture", "true" );
 	get( dropFrame, "dropFrame", "true" );
-	get( audioDevice, sizeof(audioDevice), "audioDevice", "default" );
+	get( audioDevice, sizeof(audioDevice), "audioDevice2", "/dev/dsp" );
 	get( enableV4L, "enableV4L", "false" );
 	get( disableKeyRepeat, "disableKeyRepeat", "false" );
 	get( audioRendering, "audioRendering", "0" );
@@ -669,7 +669,7 @@ Preferences::Save()
 	set( "syt_offset", syt_offset );
 	set( "preview_capture", preview_capture );
 	set( "dropFrame", dropFrame );
-	set( "audioDevice", audioDevice );
+	set( "audioDevice2", audioDevice );
 
 	/* JogShuttle Stuff */
 	set( "enableJogShuttle", enableJogShuttle );
