@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -84,6 +84,8 @@ private:
 	void JogShuttleInitModeSwitching(int& count);
 	void JogShuttleInitGeneral(int& count);
 	void JogShuttleInitTrim(int& count);
+	void RecentFilesInit();
+	void RecentFilesSave();
 
 protected:
 	Preferences();
@@ -98,6 +100,7 @@ public:
 	void set( const char *name, const char* value );
 	void set( const char *name, int value );
 	void set( const char *name, bool value );
+	void addRecentFile( string filename );
 
 	int defaultNormalisation;
 	int defaultAudio;
@@ -135,7 +138,6 @@ public:
 	bool enableV4L;
 	bool disableKeyRepeat;
 	int audioRendering;
-	int previewRendering;
 	int maxUndos;
 	int dvCaptureBuffers;
 	int dvExportBuffers;
@@ -152,7 +154,6 @@ public:
 	int displayExtract;
 	bool relativeSave;
 
-	int dvCaptureDriver;
 	char dvCaptureDevice[ 512 ];
 	bool dv1394Preview;
 	char dvExportDevice[ 512 ];
@@ -212,7 +213,6 @@ public:
 	int exportMjpegFormat;
 	bool exportMjpegSceneSplit;
 	bool exportMjpegCleanup;
-	bool exportMjpegOutputDVDAuthorXML;
 	int exportMjpegDvdTool;
 	char exportMjpegVideoPipe[512];
 	char exportMjpegAudioPipe[512];
@@ -221,6 +221,8 @@ public:
 	// Other "sticky" options
 	bool enableAVC;
 
+	vector<string> recentFiles;
+
 	void Save();
 };
 
