@@ -40,14 +40,14 @@ RWPipe::~RWPipe( )
 
 bool RWPipe::run( string command )
 {
-	char * args[ 4 ];
+	const char * args[ 4 ];
 
 	args[ 0 ] = "/bin/sh";
 	args[ 1 ] = "-c";
 	args[ 2 ] = ( char * ) command.c_str( );
 	args[ 3 ] = NULL;
 
-	return g_spawn_async_with_pipes( ".", args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, &m_reader, NULL, &m_error );
+	return g_spawn_async_with_pipes( ".", (gchar**) args, NULL, G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL, &m_pid, &m_writer, &m_reader, NULL, &m_error );
 }
 
 bool RWPipe::isRunning( )
