@@ -1,6 +1,6 @@
 /*
 * smiltime.cc -- W3C SMIL2 Time value parser
-* Copyright (C) 2003 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2003-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -181,7 +181,7 @@ string getFraction( string& time )
 	string fraction;
 	if ( ( pos = time.find( '.' ) ) != string::npos )
 	{
-		fraction = time.substr( pos );
+		fraction = time.substr( pos + 1 );
 		//cerr << "smil: fraction = " << fraction << endl;
 		time = time.substr( 0, pos );
 	}
@@ -212,7 +212,7 @@ long Time::parseClockValue( string time )
 		//cerr << "smil: parsing Partial clock value " << time << endl;
 		if ( ( pos2 = time.find( '.' ) ) != string::npos )
 		{
-			milliseconds = "0" + time.substr( pos2 ); //keep the decimal point
+			milliseconds = time.substr( pos2 + 1, 3 );
 			time = time.substr( 0, pos2 );
 		}
 		minutes = time.substr( 0, pos1 );
@@ -224,12 +224,12 @@ long Time::parseClockValue( string time )
 		//cerr << "smil: parsing Timecount value " << time << endl;
 		if ( StringUtils::ends( time, "h" ) )
 		{
-			total = ( long ) ( atof( getFraction( time ).c_str() ) * 3600000 );
+			total = ( long ) ( atof( getFraction( time ).c_str() ) * 3600 );
 			hours = time;
 		}
 		else if ( StringUtils::ends( time, "min" ) )
 		{
-			total = ( long ) ( atof( getFraction( time ).c_str() ) * 60000 );
+			total = ( long ) ( atof( getFraction( time ).c_str() ) * 60 );
 			minutes = time;
 		}
 		else if ( StringUtils::ends( time, "ms" ) )
@@ -238,7 +238,7 @@ long Time::parseClockValue( string time )
 		}
 		else
 		{
-			total = ( long ) ( atof( getFraction( time ).c_str() ) * 1000 );
+			total = atol( getFraction( time ).c_str() );
 			seconds = time;
 		}
 	}
@@ -246,7 +246,7 @@ long Time::parseClockValue( string time )
 	//	", m = " << atol(minutes.c_str()) << ", s = " << atol(seconds.c_str()) <<
 	//	", ms = " << (long) (atof(milliseconds.c_str()) * 1000) << endl;
 	total += ( atol( hours.c_str() ) * 3600 + atol( minutes.c_str() ) * 60 +
-	           atol( seconds.c_str() ) ) * 1000 + ( long ) ( atof( milliseconds.c_str() ) * 1000 );
+	           atol( seconds.c_str() ) ) * 1000 + atol( milliseconds.c_str() );
 	return total;
 }
 
