@@ -1,6 +1,6 @@
 /*
 * storyboard.cc Storyboard view object
-* Copyright (C) 2003 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2003-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "error.h"
 #include "page_editor.h"
 #include "commands.h"
+#include "stringutils.h"
 
 extern "C"
 {
@@ -176,7 +177,7 @@ on_name_show_popup( MgCellRendererList *cell,
 	int currentScene = getSceneFromPath( const_cast< char* >( path_string ) );
 
 	gtk_tree_model_get_iter (model, &iter, path);
-	string_utils::split( Preferences::getInstance().metaNames, ",", metaNames );
+	StringUtils::split( Preferences::getInstance().metaNames, ",", metaNames );
 
 	for ( metaNamesIter = metaNames.begin(); metaNamesIter != metaNames.end(); ++metaNamesIter )
 	{
@@ -219,7 +220,7 @@ on_name_edited( MgCellRendererList *cell,
 	}
 	gtk_tree_model_get_iter( GTK_TREE_MODEL( model ), &iter, path );
 	
-	string_utils::split( Preferences::getInstance().metaNames, ",", metaNames );
+	StringUtils::split( Preferences::getInstance().metaNames, ",", metaNames );
 
 
 	for ( GList *l = cell->list; l; l = l->next )
@@ -359,6 +360,16 @@ on_value_edited( MgCellRendererList *cell,
 			{
 				// Get the value from the entry
 				label = value = const_cast< char* >( mg_popup_entry_get_text( MG_POPUP_ENTRY( cell->parent.editable ) ) );
+
+				// See if a hidden value exists for the label value
+				for ( metaValuesIter = metaValues.begin(); metaValuesIter != metaValues.end(); ++metaValuesIter )
+				{
+					if ( metaValuesIter->first == label )
+					{
+						value = const_cast< char* >( metaValuesIter->second.c_str() );
+						break;
+					}
+				}
 			}
 			else if ( MG_CELL_RENDERER_POPUP( cell )->shown == TRUE )
 			{
@@ -444,8 +455,6 @@ Storyboard::Storyboard( KinoCommon *common ) :
 	GtkTreeViewColumn* column;
 	GtkTreeSelection* select;
 
-	gdk_threads_enter();
-
 	// Create the tree view
 	view = GTK_TREE_VIEW( lookup_widget( common->getWidget(), "treeview_storyboard" ) );
 	g_signal_connect( G_OBJECT( view ), "button-release-event", G_CALLBACK( tree_view_row_select ), this );
@@ -530,7 +539,6 @@ Storyboard::Storyboard( KinoCommon *common ) :
 	select = gtk_tree_view_get_selection( view );
 	gtk_tree_selection_set_mode( select, GTK_SELECTION_SINGLE );
 	
-	gdk_threads_leave();
 	g_object_unref( G_OBJECT( model ) ); // view has the reference
 }
 
@@ -587,8 +595,7 @@ void *showScenesThread( void *arg )
 			                                  NULL, NULL );
 
 			// create the thumbnail
-			// TODO: properly support widescreen
-			thumbnail = gdk_pixbuf_scale_simple( image, 80, 60, GDK_INTERP_NEAREST );
+			thumbnail = gdk_pixbuf_scale_simple( image, 80, frame.IsWide() ? 45 : 60, GDK_INTERP_NEAREST );
 			icons->push_back( thumbnail );
 			g_object_unref( image );
 		}
@@ -610,18 +617,20 @@ void *showScenesThread( void *arg )
 				char start[ 1024 ] = "";
 				vector< std::string > metaNames;
 				vector< std::string >::iterator metaNamesIter;
-				string_utils::split( Preferences::getInstance().metaNames, ",", metaNames );
+				StringUtils::split( Preferences::getInstance().metaNames, ",", metaNames );
 
 				// Generate scene info
 				frameNum = ( i == 0 ) ? 0 : scene[ i - 1 ];
 				fail_neg( common->getPlayList() ->GetFrame( frameNum, frame ) );
 				frame.ExtractHeader();
 				storyboard->getTime().setFramerate( frame.GetFrameRate() );
+				char* fileName = common->getPlayList() ->GetFileNameOfFrame( frameNum );
 				snprintf( start, 1024, "%s\n%s\n%s",
-					g_path_get_basename( common->getPlayList() ->GetFileNameOfFrame( frameNum ) ),
+					g_path_get_basename( fileName ),
 					storyboard->getTime().parseFramesToString( frameNum, common->getTimeFormat() ).c_str(),
 					storyboard->getTime().parseFramesToString( scene[ i ] - frameNum, common->getTimeFormat() ).c_str()
 				);
+				free( fileName );
 				
 				// Put the thumbnail and scene metadata into the model
 				gtk_tree_store_append( storyboard->getModel(), &iter, NULL );
