@@ -1,7 +1,6 @@
 // timfx 
-// Copyright 2002, Timothy M. Shead
-//
-// Contact: tshead@k-3d.com
+// Copyright 2002, Timothy M. Shead <tshead@k-3d.com>
+// Copyright 2006-2007 Dan Dennedy <dan@dennedy.org>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public
@@ -22,7 +21,7 @@
 #endif
 
 #include "blur.h"
-#include "kino_plugin_types.h"
+#include "../kino_plugin_types.h"
 #include "../image_filters.h"
 #include "../kino_extra.h"
 
@@ -58,11 +57,6 @@ public:
 		gtk_widget_destroy( m_window );
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-	
 	char *GetDescription( ) const
 	{
 		return _("Blur");
@@ -70,6 +64,15 @@ public:
 
 	void FilterFrame(uint8_t* pixels, int width, int height, double position, double frame_delta)
 	{
+		GtkWidget *widget = glade_xml_get_widget( m_glade, "spinbutton_blur_radius" );
+		m_radius = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( widget ) );
+
+		widget = glade_xml_get_widget( m_glade, "checkbutton_blur_horizontal" );
+		m_horizontal = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+
+		widget = glade_xml_get_widget( m_glade, "checkbutton_blur_vertical" );
+		m_vertical = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+		
 		// Create an N-pixel filter kernel
 		kino::convolve_filter<kino::basic_rgb<double> > filter;
 		
@@ -178,18 +181,6 @@ public:
 		gtk_widget_reparent( ( GTK_BIN( bin ) )->child, GTK_WIDGET( m_window ) );
 	}
 
-	void InterpretWidgets( GtkBin *bin )
-	{
-		GtkWidget *widget = glade_xml_get_widget( m_glade, "spinbutton_blur_radius" );
-		m_radius = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( widget ) );
-
-		widget = glade_xml_get_widget( m_glade, "checkbutton_blur_horizontal" );
-		m_horizontal = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
-
-		widget = glade_xml_get_widget( m_glade, "checkbutton_blur_vertical" );
-		m_vertical = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
-	}
-
 private:
 	unsigned int m_radius;
 	bool m_horizontal;
