@@ -1,4 +1,8 @@
 /*
+* v4l.cc Video4Linux management
+* Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
+*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
@@ -57,7 +61,7 @@ static int ENCODE_YUV = 0;
 
 bool V4LDevice::request( int req, V4LStruct *v4l )
 {
-	ENCODE_YUV = getenv( "KINO_V4L_PATCHED" ) != NULL;
+	ENCODE_YUV = getenv( "KINO_V4L_FFMPEG" ) != NULL;
 	return request( req, v4l->getStruct() );
 }
 
@@ -823,10 +827,7 @@ void *DvEncoder::startThread( void *ptr )
 
 void DvEncoder::writeThread()
 {
-	struct timespec tm =
-	    {
-		    0, 0
-	    };
+	struct timespec tm = { 0, 1000000 }; // 1ms
 	char command[ 10240 ];
 	int counter = 0;
 	struct stat stats;
@@ -848,7 +849,8 @@ void DvEncoder::writeThread()
 	if ( ENCODE_YUV )
 	{
 		encode = KinoVideoFactory::CreateVideoPipe( PIPE_VIDEO_DV_YUV );
-		sprintf( command, "encodedv -a dsp -i yuv422 -p 2 -q 2 - %s > %s", audio, thisfile.c_str() );
+		sprintf( command, "ffmpeg -threads 2 -f yuv4mpegpipe -i pipe: -f audio_device -ac 2 -ar 48000 -i %s -y %s", audio, thisfile.c_str() );
+//		sprintf( command, "ffmpeg -f yuv4mpegpipe -i pipe: -f audio_device -ac 2 -ar 48000 -i %s -vcodec mpeg4 -qscale 1 -aspect 4:3 -acodec pcm_s16le -y %s.avi", audio, thisfile.c_str() );
 	}
 	else
 	{
@@ -905,10 +907,7 @@ extern "C"
 			while ( v4l->active )
 			{
 				v4l->draw();
-				struct timespec t =
-				    {
-					    0, 0
-				    };
+				struct timespec t = { 0, 1000000 }; // 1ms
 				nanosleep( &t, NULL );
 			}
 		}
@@ -923,10 +922,7 @@ extern "C"
 			while ( v4l->active )
 			{
 				v4l->capture( encoder );
-				struct timespec t =
-				    {
-					    0, 0
-				    };
+				struct timespec t = { 0, 1000000 };
 				nanosleep( &t, NULL );
 			}
 			delete encoder;
